# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: timeless/timeless.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from ..jobs import job_pb2 as jobs_dot_job__pb2
from ..jobs import dag_pb2 as jobs_dot_dag__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x17timeless/timeless.proto\x12\x08timeless\x1a\x0ejobs/job.proto\x1a\x0ejobs/dag.proto\"F\n\x10RandomDAGRequest\x12\x11\n\tmax_depth\x18\x01 \x01(\x05\x12\x1f\n\x17max_children_per_parent\x18\x02 \x01(\x05\x32m\n\x08Timeless\x12\x39\n\x0cGetRandomDag\x12\x1a.timeless.RandomDAGRequest\x1a\r.jobs.dag.DAG\x12&\n\x0c\x46illJobsFlow\x12\t.jobs.Job\x1a\t.jobs.Job\"\x00\x62\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'timeless.timeless_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _RANDOMDAGREQUEST._serialized_start=69
  _RANDOMDAGREQUEST._serialized_end=139
  _TIMELESS._serialized_start=141
  _TIMELESS._serialized_end=250
# @@protoc_insertion_point(module_scope)
